﻿Friend Class MainViewModel

	Private FormFactory As IFormFactory = Nothing
	Private WithEvents Form As IMainForm = Nothing
	Private WithEvents Game As Game = Nothing
	Private GameFileContentManager As GameFileContentManager = Nothing
	Private ReadOnly SettingsFilePath As String = "Settings.txt"
	Private ReadOnly DefaultStatisticsFilePath As String = "Ergebnisse.txt"

	Public Sub New(FormFactory As IFormFactory)
		RememberFormFactory(FormFactory)
	End Sub

	Private Sub RememberFormFactory(FormFactory As IFormFactory)
		Me.FormFactory = FormFactory
	End Sub

	Public Sub StartGame()
		EnsureTheExistingOfAValidSettingsFileContaining(DefaultStatisticsFilePath)
		CreateGameFileContentManager()
		CreateNewMainForm()
		ImportData()
		WireEvents()
		ShowCurrentGameStatistics()
		ShowGameForm()
		CutEventWires()
	End Sub

	Private Sub EnsureTheExistingOfAValidSettingsFileContaining(DefaultStatisticsFilePath As String)
		If Not IO.File.Exists(SettingsFilePath) OrElse GetFilePathOfStatisticsFile.Count = 0 Then WriteStatisticsFilePathIntoSettingsFile(SettingsFilePath, DefaultStatisticsFilePath)
	End Sub

	Private Function GetFilePathOfStatisticsFile() As String
		Return IO.File.ReadAllText(SettingsFilePath)
	End Function

	Private Sub WriteStatisticsFilePathIntoSettingsFile(SettingsFilePath As String, StatisticsFilePath As String)
		IO.File.WriteAllText(SettingsFilePath, StatisticsFilePath)
	End Sub

	Private Sub CreateGameFileContentManager()
		GameFileContentManager = New GameFileContentManager(GetFilePathOfStatisticsFile())
	End Sub

	Private Sub CreateNewMainForm()
		Me.Form = FormFactory.CreateNewMainForm
	End Sub

	Private Sub ImportData()
		Dim CurrentGameStatistics = GameFileContentManager.GetGameStatisticsFromFile()
		Game = New Game(CurrentGameStatistics)
	End Sub

	Private Sub WireEvents()
		AddHandler Form.ThrowDice, AddressOf ThrowDice
		AddHandler Form.ShowConfigurations, AddressOf ShowConfigurations
		AddHandler Form.SaveAndExit, AddressOf EndGame
	End Sub

	Private Sub ShowCurrentGameStatistics()
		Form.ShowPlayerVictoryCount(Game.PlayerVictoryCount.ToString)
		Form.ShowComputerVictoryCount(Game.ComputerVictoryCount.ToString)
		Form.ShowDrawCount(Game.DrawCount.ToString)
	End Sub

	Private Sub ShowGameForm()
		Form.Show()
	End Sub

	Private Sub ShowConfigurations(sender As Object, e As EventArgs)
		Dim ConfigurationsViewModel As New ConfigurationsViewModel(FormFactory, GetFilePathOfStatisticsFile)
		ConfigurationsViewModel.ShowConfigurations()
		If ConfigurationsViewModel.NewFilePath = GetFilePathOfStatisticsFile() Then Return

		WriteStatisticsFilePathIntoSettingsFile(SettingsFilePath, ConfigurationsViewModel.NewFilePath)
		CreateGameFileContentManager()
	End Sub

	Private Sub CutEventWires()
		RemoveHandler Form.ThrowDice, AddressOf ThrowDice
		RemoveHandler Form.ShowConfigurations, AddressOf ShowConfigurations
		RemoveHandler Form.SaveAndExit, AddressOf EndGame
	End Sub

	Public Sub ThrowDice(sender As Object, e As EventArgs)
		Game.ThrowDice()
	End Sub

	Private Sub EndGame(sender As Object, e As EventArgs)
		SaveStatistic()
		CloseForm()
	End Sub

	Private Sub CloseForm()
		Form.Close()
	End Sub

	Public Sub SaveStatistic()
		GameFileContentManager.WriteGameStatisticsToFile((Game.PlayerVictoryCount, Game.ComputerVictoryCount, Game.DrawCount))
	End Sub

	Private Sub Game_DiceThrow(sender As Object, e As DiceThrowingEventArgs) Handles Game.DiceThrow
		ShowResults(e)
		ShowCurrentGameStatistics()
	End Sub

	Private Sub ShowResults(e As DiceThrowingEventArgs)
		Form.ShowPlayerResult(e.PlayerResult.ToString)
		Form.ShowComputerResult(e.ComputerResult.ToString)
		Dim Result As String = Nothing
		Select Case e.Result
			Case ThrowResult.PlayerWon : Result = "menno …"
			Case ThrowResult.ComputerWon : Result = "SIEG!"
			Case ThrowResult.Draw : Result = "langweilig"
		End Select
		Form.ShowResult(Result)
	End Sub

End Class